#!/usr/bin/env python3

import numpy as np
import ezdxf 
import random
from math import sqrt
import copy
unit_convert=10000 #ses to dxf


def cross_cut(msp,cross_size,pin_loc):
    """
    cross_size= lenght of bounding box of the cross cut
    """
    tipat=cross_size/2
    for pt in pin_loc:
        x=pt[0]
        y=pt[1]
        cross_s1=(x-tipat,y-tipat)
        cross_e1=(x+tipat,y+tipat)
        cross_s2=(x-tipat,y+tipat)
        cross_e2=(x+tipat,y-tipat)
        msp.add_line(cross_s1,cross_e1,dxfattribs={'linetype':'DASHDOT', 'layer':'Circuit'})       
        msp.add_line(cross_s2,cross_e2,dxfattribs={'linetype':'DASHDOT', 'layer':'Circuit'})
    
# def line_to_path()

class post_process():
    def __init__(self,dxf_file,wiring_path):
        if not dxf_file.endswith('.dxf'):
            dxf_file+='.dxf'
        self.dxf_file=dxf_file
        self._wiring=wiring_path
        self.wiring=[]
        self.dwg=ezdxf.readfile(dxf_file)
        self.msp=self.dwg.modelspace()

        self.convert_unitfrom_ses()
        # self.draw_wires()
        self.parallel_trace(1)

        self.dwg.saveas('route_text.dxf')

    def convert_unitfrom_ses(self):
        for path in self._wiring:
            p=np.array(path)/unit_convert
            p[:,1]*=-1
            self.wiring.append(p.tolist())         
    def draw_wires(self):
        for path in self.wiring:
            for i in range(len(path)-1):
                self.msp.add_line(path[i],path[i+1],dxfattribs={
                    'layer':'Circuit',
                    'linetype':'DASHDOT'})

    def parallel_trace(self,width):
        halfw=width/2
        for path in self.wiring:
            for i in range(len(path)-1):
                start_x,start_y=path[i][0],path[i][1]
                end_x,end_y=path[i+1][0],path[i+1][1]
                
                theta=self.find_lind_eq(path[i],path[i+1])[3]
                sinw=np.sin(theta)*halfw
                cosw=np.cos(theta)*halfw
                s1i=[start_x-sinw,start_y+cosw]
                s1o=[start_x+sinw,start_y-cosw]                
                e1i=[end_x-sinw,end_y+cosw]
                e1o=[end_x+sinw,end_y-cosw]

                line1i=self.find_lind_eq(s1i,e1i)
                line1o=self.find_lind_eq(s1o,e1o)

                if i==0:
                    s1=s1i
                    s2=s1o
                else:
                    s1=e1
                    s2=e2 
                
                if i<len(path)-2:
                    next_x,next_y=path[i+2][0],path[i+2][1]
                    theta2=self.find_lind_eq(path[i+1],path[i+2])[3]
                   
                    sinw2=np.sin(theta2)*halfw
                    cosw2=np.cos(theta2)*halfw

                    s2i=[end_x-sinw2,end_y+cosw2]
                    s2o=[end_x+sinw2,end_y-cosw2]
                    e2i=[next_x-sinw2,next_y+cosw2]
                    e2o=[next_x+sinw2,next_y-cosw2]

                    line2i=find_lind_eq(s2i,e2i)
                    line2o=find_lind_eq(s2o,e2o)

                    Ainteri=np.array([[line1i[0],-line1i[2]],[line2i[0],-line2i[2]]])
                    binteri=np.array([-line1i[1],-line2i[1]])
                    Aintero=np.array([[line1o[0],-line1o[2]],[line2o[0],-line2o[2]]])
                    bintero=np.array([-line1o[1],-line2o[1]])
                    
                    e1=np.linalg.solve(Ainteri,binteri)
                    e2=np.linalg.solve(Aintero,bintero)

                else:
                    #from last end point to this end
                    e1=[end_x-sinw,end_y+cosw]
                    e2=[end_x+sinw,end_y-cosw]
                

                self.msp.add_line(s1i,e1i,dxfattribs={
                    'layer':'Circuit',
                    'linetype':'DASHDOT'})
                self.msp.add_line(s1o,e1o,dxfattribs={
                    'layer':'Circuit',
                    'linetype':'DASHDOT'})

def find_line_eq(pt1,pt2):
    """
    input: two points on a line
    pt1:[x1,y1]
    pt2:[x2,y2]
    return: a,b,theta in terms of cy=ax+b
    """
    
    x1,y1=pt1[0],pt1[1]
    x2,y2=pt2[0],pt2[1]
    if x1>x2:
        x1,y1=pt2[0],pt2[1]
        x2,y2=pt1[0],pt1[1]

    if x1==x2:
        theta=-np.pi/2
        #x= number
        a=1
        b=-x1
        c=0
    else:
        theta=np.arctan((y2-y1)/(x2-x1))
        A=np.array([[x1,1],[x2,1]])
        k=np.array([y1,y2])
        sol=np.linalg.solve(A,k)
        a,b=sol[0],sol[1]
        c=1

    return a,b,c,theta

def find_angle_bisector(line_eq1,line_eq2):
    """
    line_equation follows same pattern as find_line_eq()
    eq=[a,b,c,theta]
    which refers to cy=ax+b
    """
    